/*
 * Copyright © 2011-2012 Philipp Eichhorn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package lombok.ast;

import static lombok.ast.Modifier.*;
import lombok.*;

@Getter
public final class FieldDecl extends AbstractVariableDecl<FieldDecl> {
	private Expression<?> initialization;

	public FieldDecl(final TypeRef type, final String name) {
		super(type, name);
	}

	public TypeRef getType() {
		return type;
	}

	public FieldDecl makePublic() {
		return withModifier(PUBLIC);
	}

	public FieldDecl makePrivate() {
		return withModifier(PRIVATE);
	}

	public FieldDecl makeStatic() {
		return withModifier(STATIC);
	}

	public FieldDecl makeVolatile() {
		return withModifier(VOLATILE);
	}

	public FieldDecl makeTransient() {
		return withModifier(TRANSIENT);
	}

	public FieldDecl withInitialization(final Expression<?> initialization) {
		this.initialization = child(initialization);
		return this;
	}

	@Override
	public <RETURN_TYPE, PARAMETER_TYPE> RETURN_TYPE accept(final ASTVisitor<RETURN_TYPE, PARAMETER_TYPE> v, final PARAMETER_TYPE p) {
		return v.visitFieldDecl(this, p);
	}
}
